/*
editeurconfig - Editeur de Fichiers de Config de Jaxe

Copyright (C) 2011 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package editeurconfig;

import jaxe.InterfaceSchema;
import jaxe.wxs.JaxeWXS;
import jaxe.SchemaRelaxNG;
import jaxe.SchemaSimple;
import jaxe.Config;
import jaxe.JaxeException;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.border.*;
import java.util.*;
import java.net.URL;
import org.w3c.dom.*;
import java.io.*;
import javax.swing.filechooser.FileFilter;
import java.net.MalformedURLException;


/**
* Reprsente la liste des lments
*/
public class GenerationAutomatique {
    
    private EditeurFichierConfig appl;
    private InterfaceSchema schema;
    private Document doc;
    private Element racine;
    private Element langage;
    private Element fichier_schema;
    private Element menus;
    private Element affichage_noeuds;
    
    
    /**
    * Gnre un fichier de config  partir d'un schma
    */
    public GenerationAutomatique(final EditeurFichierConfig appl, final Document doc, final Element racine) {
        
        this.appl = appl;
        this.doc = doc;
        this.racine = racine;
        
        parcourir();
    }
    
    
    private void parcourir() {
        File f = null;
        if (System.getProperty("os.name").indexOf("Linux") != -1 && System.getProperty("java.version").compareTo("1.7") < 0) {
            JFileChooser jfc = new JFileChooser();
            jfc.setFileFilter(new FileFilter() {
                public boolean accept(File f) {
                    if (f.isDirectory())
                        return true;
                    String s = f.getName();
                    int i = s.lastIndexOf('.');

                    if (i > 0 && i < s.length() - 1)
                        if (s.substring(i + 1).toLowerCase().equals("xsd") || s.substring(i + 1).toLowerCase().equals("rng"))
                            return true;

                    return false;
                }

                public String getDescription() {
                    return Strings.get("texte.FichiersSchema");
                }
            });
            
            if (jfc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION)
                f = jfc.getSelectedFile();
        }
        else {
            final FileDialog fd = new FileDialog(new JFrame(), null, FileDialog.LOAD);
            //fd.setFile("*.xsd");
            fd.setVisible(true);
            final String sf = fd.getFile();
            if (sf != null)
                f = new File(fd.getDirectory(), sf);
        }
        
        if (f != null) {
            if (f.getName().toLowerCase().endsWith(".xsd") || f.getName().toLowerCase().endsWith(".rng"))
                charger(f);
            else {
                JOptionPane.showMessageDialog(null, Strings.get("message.PasUnSchema"), Strings.get("titre.Erreur"), JOptionPane.ERROR_MESSAGE);
                new EditeurFichierConfig();
            }
        }
        else 
            new EditeurFichierConfig();
            
    }
    
    
    private void charger(final File f) {
        
        ListeElements listeElements = null;
        ArrayList<Element> tabRefEl = null;
        ArrayList<String> lesElements = null;
        try {
            URL urlFichier = f.toURI().toURL();
            listeElements = new ListeElements(urlFichier, null, null);
        } catch(MalformedURLException ex) {
            System.out.println("Erreur urlFichier "+ex);
        }
        
        schema = listeElements.getInterfaceSchema();
        
        // LANGAGE
        enregistrerElementLangage();
        // FICHIER_SCHEMA
        enregistrerElementSchema(f);
        
        // RACINE
        ArrayList<String> nomsElementsRacines = null;
        nomsElementsRacines = listeElements.getNomsElementsRacinesP(true);
        for (int i = 0; i< nomsElementsRacines.size();i++)
            enregistrerElementRacine(nomsElementsRacines.get(i));
            
            
        // MENUS
        enregistrerMenus();
        
        // MENU
        Element menu1 = enregistrerMenu("Blocs");
        Element menu2 = enregistrerMenu("Texte");
        Element menu3 = enregistrerMenu("Autre");

        // AFFICHAGE_NOEUDS
        enregistrerAffichageNoeuds();
        
        tabRefEl = listeElements.getReferencesElements();
        
        // MENU_INSERTION
        for (int i = 0; i< tabRefEl.size(); i++) {
            String nomElement = schema.nomElement(tabRefEl.get(i));
            if (listeElements.elementDeBloc(tabRefEl.get(i)))
                enregistrerMenuInsertion(menu1, nomElement);
            else if (listeElements.elementMelangeTexte(tabRefEl.get(i)))
                enregistrerMenuInsertion(menu2, nomElement);
            else
                enregistrerMenuInsertion(menu3, nomElement);
        }
            
        
        // AFFICHAGE_ELEMENT
        for (int i = 0; i< tabRefEl.size(); i++) {
            String nomElement = schema.nomElement(tabRefEl.get(i));
            
            if (nomsElementsRacines.contains(schema.nomElement(tabRefEl.get(i))))
                enregistrerAffichageElement(nomElement, "division");
            else {
                if (schema.listeSousElements(tabRefEl.get(i)).size() == 0) {
                    if (schema.contientDuTexte(tabRefEl.get(i))) {
                        if (schema.listeValeursElement(tabRefEl.get(i)) != null)
                            enregistrerAffichageElement(nomElement, "typesimple");
                    }
                    else
                        enregistrerAffichageElement(nomElement, "vide");
                }
                if (schema.contientDuTexte(tabRefEl.get(i)) || schema.listeSousElements(tabRefEl.get(i)).size() > 0) {
                    if (listeElements.parentContientDuTexte(tabRefEl.get(i)))
                        enregistrerAffichageElement(nomElement, "string");
                    else
                        enregistrerAffichageElement(nomElement, "zone");
                }
            }
        }
        
        
        // STRINGS
        String langue = Locale.getDefault().getLanguage();
        Element strings = enregistrerElementStrings(langue);
        
        
        // STRINGS_ELEMENT
        for (int i = 0; i< tabRefEl.size(); i++) {
            String nomElement = schema.nomElement(tabRefEl.get(i));
            Element strings_element = enregistrerStringsElement(strings, nomElement);
            
            enregistrerTitre(strings_element, nomElement);
            if (schema.documentationElement(tabRefEl.get(i)) != null)
                enregistrerDocumentation(strings_element, schema.documentationElement(tabRefEl.get(i)).trim());
            
            // STRINGS_ATTRIBUT
            ArrayList<Element> listeAttributs = schema.listeAttributs(tabRefEl.get(i));
            if (listeAttributs != null && listeAttributs.size() > 0) {
                for (int j = 0; j< listeAttributs.size(); j++) {
                    String nomAttribut = schema.nomAttribut(listeAttributs.get(j));
                    Element strings_attribut = enregistrerStringsAttribut(strings_element, nomAttribut);
                    enregistrerTitre(strings_attribut, nomAttribut);
                    if (schema.documentationAttribut(listeAttributs.get(j)) != null)
                        enregistrerDocumentation(strings_attribut, schema.documentationAttribut(listeAttributs.get(j)).trim());
                }
            }
        }
        
        new FenetrePrincipale(appl, null, doc, racine, listeElements);
    }
    
    
    
    /**
    * Enregistre l'lment LANGAGE dans le DOM
    */
    private void enregistrerElementLangage() {
        langage = doc.createElement("LANGAGE");
        racine.appendChild(langage);
    }
    
    /**
    * Enregistre l'lment FICHIER_SCHEMA dans le DOM
    */
    private void enregistrerElementSchema(final File f) {
        fichier_schema = doc.createElement("FICHIER_SCHEMA");
        fichier_schema.setAttribute("nom", f.getName());
        langage.appendChild(fichier_schema);
        
        EditeurFichierConfig.setModif(true);
    }
    
    /**
    * Enregistre l'lment RACINE dans le DOM
    */
    private void enregistrerElementRacine(final String racineE) {
        //Element elementRacine = Outils.getElementSelectionne(langage, "RACINE", racineE, "element");
        //if (elementRacine == null) {
        Element elementRacine = doc.createElement("RACINE");
        langage.appendChild(elementRacine);
        elementRacine.setAttribute("element", racineE);
        
        EditeurFichierConfig.setModif(true);
        //}
    }
    
    
    /**
    * Enregistre l'lment MENUS dans le DOM
    */
    private Element enregistrerMenus() {
        menus = doc.createElement("MENUS");
        racine.appendChild(menus);
        return menus;
    }
    
    /**
    * Enregistre les lments MENU dans le DOM
    */
    private Element enregistrerMenu(final String nomM) {
        Element menu = doc.createElement("MENU");
        menus.appendChild(menu);
        menu.setAttribute("nom", nomM);
        return menu;
    }
    
    /**
    * Enregistre les lments MENU_INSERTION dans le DOM
    */
    private Element enregistrerMenuInsertion(final Node menu, final String nom) {
        Element menu_insertion = doc.createElement("MENU_INSERTION");
        menu.appendChild(menu_insertion);
        menu_insertion.setAttribute("nom", nom);
        return menu_insertion;
    }
    
    /**
    * Enregistre les lments AFFICHAGE_NOEUDS dans le DOM
    */
    private Element enregistrerAffichageNoeuds() {
        affichage_noeuds = doc.createElement("AFFICHAGE_NOEUDS");
        affichage_noeuds = doc.createElement("AFFICHAGE_NOEUDS");
        racine.appendChild(affichage_noeuds);
        return affichage_noeuds;
    }
    
    /**
    * Enregistre les lments AFFICHAGE_ELEMENT dans le DOM
    */
    private void enregistrerAffichageElement(final String nomElement, final String type) {
        Element affichage_element = doc.createElement("AFFICHAGE_ELEMENT");
        affichage_noeuds.appendChild(affichage_element);
        affichage_element.setAttribute("element", nomElement);
        affichage_element.setAttribute("type", type);
        
        EditeurFichierConfig.setModif(true);
    }
    
    
    /**
    * Enregistre les lments STRINGS dans le DOM
    */
    private Element enregistrerElementStrings(final String langue) {
        Element strings = doc.createElement("STRINGS");
        racine.appendChild(strings);
        strings.setAttribute("langue", langue);
        return strings;
    }
    
    /**
    * Enregistre les lments STRINGS_ELEMENT dans le DOM
    */
    private Element enregistrerStringsElement(final Element strings, final String nomElement) {
        Element strings_element = doc.createElement("STRINGS_ELEMENT");
        strings.appendChild(strings_element);
        strings_element.setAttribute("element", nomElement);
        EditeurFichierConfig.setModif(true);
        return strings_element;
    }
    
    /**
    * Enregistre les lments STRINGS_ATTRIBUT dans le DOM
    */
    private Element enregistrerStringsAttribut(final Element strings_element, final String nomAttribut) {
        Element strings_attribut = doc.createElement("STRINGS_ATTRIBUT");
        strings_element.appendChild(strings_attribut);
        strings_attribut.setAttribute("attribut", nomAttribut);
        EditeurFichierConfig.setModif(true);
        return strings_attribut;
    }
    
    /**
    * Enregistre les lments TITRE dans le DOM
    */
    private void enregistrerTitre(final Element element, final String texteTitre) {
        Element titre = Outils.premierEnfantDeNom(element, "TITRE");
        if (titre == null) {
            titre = doc.createElement("TITRE");
            element.appendChild(titre);
        }
        Outils.setValeurElement(doc, titre, texteTitre);
    }
    
    /**
    * Enregistre les lments DOCUMENTATION dans le DOM
    */
    private void enregistrerDocumentation(final Element element, final String texteDocumentation) {
        Element documentation = Outils.premierEnfantDeNom(element, "DOCUMENTATION");
        if (documentation == null) {
            documentation = doc.createElement("DOCUMENTATION");
            element.appendChild(documentation);
        }
        Outils.setValeurElement(doc, documentation, texteDocumentation);
    }
    
}
