/*
editeurconfig - Editeur de Fichiers de Config de Jaxe

Copyright (C) 2011 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package editeurconfig;

import java.util.*;

public class ElementSS implements java.lang.Comparable {

    private String nom;
    private String texte;
    private ArrayList<ElementSS> parents;
    private ArrayList<ElementSS> sousElements;
    private ArrayList<EnsembleSS> sousEnsembles;
    private ArrayList<AttributSS> attributs;
    

    public ElementSS(String nom) {
        this.nom = nom;
        texte = "";
        parents = new ArrayList<ElementSS>();
        sousElements = new ArrayList<ElementSS>();
        sousEnsembles = new ArrayList<EnsembleSS>();
        attributs = new ArrayList<AttributSS>();
    }
    
    
    /******* GET **************/
    public String getNom() {
        return nom;
    }
    
    public String getTexte() {
        return texte;
    }
    
    public ArrayList<ElementSS> getParents() {
        Collections.sort(parents);
        return parents;
    }
    
    public ArrayList<ElementSS> getSousElements() {
        Collections.sort(sousElements);
        return sousElements;
    }
    
    public ArrayList<EnsembleSS> getSousEnsembles() {
        Collections.sort(sousEnsembles);
        return sousEnsembles;
    }
    
    public ArrayList<AttributSS> getAttributs() {
        //Collections.sort(attributs);
        return attributs;
    }
    
    
    
    
    /******* SET **************/
    public void setNom(String unNom) {
        nom = unNom;
    }
    
    public void setTexte(String unTexte) {
        texte = unTexte;
    }
    
    
    
    /**********Ajout Suppression **********************/
    // Parents
    public void ajouterParent(ElementSS unParent) {
        if (!parents.contains(unParent))
            parents.add(unParent);
    }
    
    public void supprimerParent(ElementSS unParent) {
        if (parents.contains(unParent))
            parents.remove(unParent); 
    }
    
    // Sous-Elements
    public void ajouterSousElement(ElementSS unElement) {
        if (!sousElements.contains(unElement))
            sousElements.add(unElement);
    }
    
    public void supprimerSousElement(ElementSS unElement) {
        if (sousElements.contains(unElement))
            sousElements.remove(unElement); 
    }
    
    // Sous-Ensembles
    public void ajouterSousEnsemble(EnsembleSS unEnsemble) {
        if (!sousEnsembles.contains(unEnsemble))
            sousEnsembles.add(unEnsemble); 
    }
    
    public void supprimerSousEnsemble(EnsembleSS unEnsemble) {
        if (sousEnsembles.contains(unEnsemble))
            sousEnsembles.remove(unEnsemble); 
    }
    
    // Attributs
    public void ajouterAttribut(AttributSS unAttribut) {
        if (!attributs.contains(unAttribut))
            attributs.add(unAttribut);
    }
    
    public void supprimerAttribut(AttributSS unAttribut) {
        if (attributs.contains(unAttribut))
            attributs.remove(unAttribut); 
    }
    
    
    /***************************************************/
    public String toString() {
        return getNom();
    }
    
    public int compareTo(Object ess) { 
        String nom1 = ((ElementSS)ess).getNom(); 
        String nom2 = this.getNom(); 
        if (nom2.compareTo(nom1) < 0) 
            return -1; 
        else if (nom2.compareTo(nom1) == 0) 
            return 0; 
        else 
            return 1; 
   }
}
